/**
* @author Dirk Nordmann
* @date   14.01.2002
*/
package xml2rtf.xml;

import xml2rtf.templates.*;
import util.*;
import java.util.StringTokenizer;

public class Tag 
{

public static final int NO_VALUE = 0xfff0bdc1;
public String _tag;


		
/* ######################################################################
   ###  CONSTRUCTOR
   ###
   ###  IN:  tag		Full tag content (e.g. <A HREF="url">)
   ###  OUT: Tag		New instance of Tag
   ######################################################################
 */
public Tag()
{
	_tag = null;
}

public Tag(String tag)
{
	setTag(tag);
}



//*****  Tag  *****
//--- SET
public void setTag(String tag)
{
	tag = tag.trim();
	tag = tag.replace('\n', ' ');
	if (tag.startsWith("<"))
		tag = tag.substring(1);
	if (tag.endsWith(">"))
		tag = tag.substring(0,tag.length()-1);
	_tag = tag.trim();
}		
//--- GET
public String getTag()
{
	return "<" + _tag + ">";
}		
		
		
		
/* ######################################################################
   ###  Tag.getTagName()
   ###
   ###  IN:  None
   ###  OUT: String		Tag-Name (in Uppercase). Returns null if Tag-Name
   ###					is empty.
   ######################################################################
 */
public String getTagName()
{
	StringTokenizer stringtokenizer;
	for (stringtokenizer = new StringTokenizer(_tag, "= \t\"'", true); stringtokenizer.hasMoreTokens(); )
	{
		String rtn = stringtokenizer.nextToken().toUpperCase();
		if (isEndTag())
			rtn = rtn.substring(1);
		return rtn;
	}
	return null;
}

    
public boolean isStartTag()
{
	return !isEndTag();
}
    
public boolean isEndTag()
{
	return _tag.startsWith("/");
}
    
public boolean isComment()
{
	return _tag.startsWith("!--") && _tag.endsWith("--");
}
    
public boolean isCdata()
{
	return _tag.toUpperCase().startsWith("![CDATA[") && _tag.toUpperCase().endsWith("]]");
}
    
    
/* ######################################################################
   ###  Tag.getNumericTagAttr(String attr)
   ###
   ###  IN:  attr			Attribute-Name
   ###  OUT: int			Attribute-Value. Returns NO_VALUE if Attribute is
   ###						not numeric or empty.
   ######################################################################
 */
public int getNumericTagAttr(String attr)
{
	int rtn = NO_VALUE;
	String s = getTagAttr(attr);
	if(s != null)
		try
		{
			rtn = Integer.parseInt(s);
		}
		catch (Exception _ex) 
		{ 
		}
	return rtn;
}

    
    
/* ######################################################################
   ###  Tag.getColorAttr(String attr)
   ###
   ###  IN:  attr			Attribute-Name
   ###  OUT: int			Attribute-Color. Returns NO_VALUE if Attribute is
   ###								not found or empty.
   ######################################################################
 */
public int getColorAttr(String attr)
{
    String s = getTagAttr(attr);
    if(s != null)
        return ConversionTemplate.decodeColor(s);
    else
		return NO_VALUE;
}

    
    
/* ######################################################################
   ###  Tag.getTagAttr(String attr)
   ###
   ###  IN:  attr		Attribute-Name
   ###  OUT: String		Attribute-Value. Returns null if Attribute is
   ###					not found or empty.
   ######################################################################
 */
public String getTagAttr(String attr)
{
	String rtn = "";
	attr = attr.toUpperCase();
	StringTokenizer stringtokenizer;
	boolean brflag = false;
        
	//-----  Attribute name  -----
	for(stringtokenizer = new StringTokenizer(_tag, "= \t\"'", true); 
		!brflag && stringtokenizer.hasMoreTokens(); 
		brflag = stringtokenizer.nextToken().toUpperCase().equals(attr));
	if(!brflag)
		return null;
	//-----  = (equals)  -----
	for(brflag = false; 
		!brflag && stringtokenizer.hasMoreTokens(); 
		brflag = stringtokenizer.nextToken().equals("="));
	if(!brflag)
		return null;
            
	//-----  Attribute value  -----
	brflag = false;
	while (stringtokenizer.hasMoreTokens()) 
	{
		String s = stringtokenizer.nextToken();
		if (s.equals("\"") || s.equals("'"))
		{
			brflag = !brflag;
			continue;
		}
		if(s.equals(" ") || s.equals("/") || s.equals(">"))
		{
			if(!brflag && rtn.length() > 0)
				break;
			rtn += s;
		} else
			rtn += s;
	}
        
	rtn = StringUtil.replace(rtn,"&amp;","&");
	rtn = StringUtil.replace(rtn,"&lt;","<");
        
	return rtn;
}

}