###################################################################################################
# _globals.py
#
# Implementation of Global Functions.
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import Persistent
from Acquisition import Implicit
import urllib
import string
import copy
import time

""" Globals. """

# DEPRECATED!
class MyDict(Persistent,Implicit): pass    

# Constants.
# ----------

# Debugging.
debug = 0

# Booleans.
true = 1
false = 0

# Datatypes.
DT_UNKNOWN = -1
DT_INT = 0
DT_BOOLEAN = 1
DT_STRING = 2
DT_TEXT = 4
DT_IMAGE = 5
DT_FILE = 6
DT_DATE = 7
DT_DICTIONARY = 8
DT_LIST = 9
DT_URL = 10
DT_TIME = 11
DT_DATETIME = 12
DT_FLOAT = 13
DT_PASSWORD = 14
DT_STRINGS = [DT_STRING,DT_TEXT,DT_URL,DT_PASSWORD]
DT_BLOBS = [DT_IMAGE,DT_FILE]
DT_INTS = [DT_INT,DT_BOOLEAN]
DT_NUMBERS = [DT_INT,DT_FLOAT]
DT_DATETIMES = [DT_DATE,DT_TIME,DT_DATETIME]

dtMapping = [
  'int',DT_INT,
  'float',DT_FLOAT,
  'boolean',DT_BOOLEAN,
  'string',DT_STRING,
  'text',DT_TEXT,
  'image',DT_IMAGE,
  'file',DT_FILE,
  'date',DT_DATE,
  'time',DT_TIME,
  'datetime',DT_DATETIME,
  'dictionary',DT_DICTIONARY,
  'list',DT_LIST,
  'url',DT_URL,
  'password',DT_PASSWORD
]

# Language Inheritance.
LI_FORBIDDEN = -1
LI_OBLIGATE = 0
LI_ALLOWED = 1

# -------------------------------------------------------------------------------------------------
# datatype_key:
# -------------------------------------------------------------------------------------------------
def datatype_key(datatype):
  if datatype in dtMapping:
    return dtMapping[dtMapping.index(datatype)+1]
  else:
    return DT_UNKNOWN


# -------------------------------------------------------------------------------------------------
# format_sort_id:
# -------------------------------------------------------------------------------------------------
def format_sort_id(i_sort_id):
  s_sort_id = "0000%i"%i_sort_id
  s_sort_id = "s" + s_sort_id[-4:]
  return s_sort_id


# -------------------------------------------------------------------------------------------------
# html_quote:
# -------------------------------------------------------------------------------------------------
def html_quote(v, name='(Unknown name)', md={}):
  from cgi import escape
  return escape(str(v), 1)


# -------------------------------------------------------------------------------------------------
# id_quote:
# -------------------------------------------------------------------------------------------------
def id_quote(srctext):
  dsttext = srctext
  for ch in range(0,255):
    if ch <= 32 or ch in [34,60,62] or ch >= 127:
      dsttext = string.replace(dsttext,chr(ch),'')
  return dsttext


# -------------------------------------------------------------------------------------------------
# id_prefix:
# -------------------------------------------------------------------------------------------------
def id_prefix(srctext): 
  i = 0
  while (i < len(srctext) and srctext[i] not in ['0','1','2','3','4','5','6','7','8','9']):
    i = i + 1
  return srctext[:i]


# -------------------------------------------------------------------------------------------------
# js_quote:
#
# Quote String for JavaScript.
# -------------------------------------------------------------------------------------------------
def js_quote(srctext):
  dsttext = srctext
  dsttext = string.replace(dsttext,"'","&quot;")
  dsttext = string.replace(dsttext,'"','&quot;')
  return dsttext


# -------------------------------------------------------------------------------------------------
# qs_append:
#
# Append to Query-String.
# -------------------------------------------------------------------------------------------------
def qs_append(qs, p, v):
  if len(qs) == 0:
    qs = qs + '?'
  else:
    qs = qs + '&'
  qs = qs + p + '=' + urllib.quote(v)
  return qs


# -------------------------------------------------------------------------------------------------
# nvl:
#
# Returns its first argument if it is not None, otherwise it returns its second argument.
# -------------------------------------------------------------------------------------------------
def nvl(a1, a2):
  if a1 is not None:
    return a1
  else:
    return a2


# -------------------------------------------------------------------------------------------------
# isPreviewRequest:
# -------------------------------------------------------------------------------------------------
def isPreviewRequest(REQUEST):
  return REQUEST is not None and \
         REQUEST.get('preview','') == 'preview' and \
         REQUEST.get('live','') == ''


# -------------------------------------------------------------------------------------------------
# isCacheRequest:
# -------------------------------------------------------------------------------------------------
def isCacheRequest(REQUEST):
  return not isPreviewRequest(REQUEST) and REQUEST.get('__cache__',0) == 1


# -------------------------------------------------------------------------------------------------
#  getDateTime:
# -------------------------------------------------------------------------------------------------
def getDateTime(t):
  # Patch:
  # ==========================================================================
  # [Author]: Dirk Mueller <dmuell@gmx.net>
  # [Date]:   2002-10-15
  # Bei Python 2.2 ist der Typ der Objekte des Moduls "time" nicht
  # mehr "tuple", sondern "time.struct_time". Es verhlt sich aber weiterhin
  # abwrtskompatibel zu einem tuple.
  # Fr Zope ist das kein Problem, da Zope den eigenen, etwas flexibleren
  # Datentyp DateTime verwendet. ZMS verlsst sich jedoch darauf, dass
  # der typ "tuple" ist.
  try:
    if not (type(t) is tuple or type(t) is time.struct_time):
      t = time.localtime(t)
  except:
    try:
      if not (type(t) is type(())):
        t = time.localtime(t)
    except:
      pass
  return t


# -------------------------------------------------------------------------------------------------
# compareDate:
# -------------------------------------------------------------------------------------------------
def compareDate(t0, t1, accuracy_time=1):
  YEAR = 0
  MONTH = 1
  DAY = 2
  HOURS = 3
  MINUTES = 4
  SECONDS = 5
  t0 = getDateTime(t0)
  y0 = t0[YEAR]
  m0 = t0[MONTH]
  d0 = t0[DAY]
  H0 = t0[HOURS]
  M0 = t0[MINUTES]
  S0 = t0[SECONDS]
  t1 = getDateTime(t1)
  y1 = t1[YEAR]
  m1 = t1[MONTH]
  d1 = t1[DAY]
  H1 = t1[HOURS]
  M1 = t1[MINUTES]
  S1 = t1[SECONDS]
  if (y1 > y0) or \
     (y1 == y0 and m1 > m0) or \
     (y1 == y0 and m1 == m0 and d1 > d0) or \
     (accuracy_time == 1 and y1 == y0 and m1 == m0 and d1 == d0 and H1 > H0) or \
     (accuracy_time == 1 and y1 == y0 and m1 == m0 and d1 == d0 and H1 == H0 and M1 > M0) or \
     (accuracy_time == 1 and y1 == y0 and m1 == m0 and d1 == d0 and H1 == H0 and M1 == M0 and S1 > S0):
    return +1
  elif (accuracy_time == 0 and y1 == y0 and m1 == m0 and d1 == d0) or \
       (accuracy_time == 1 and y1 == y0 and m1 == m0 and d1 == d0 and H1 == H0 and M1 == M0 and S1 == S0):
    return 0
  else:
    return -1


###################################################################################################
###################################################################################################
###
###   MyDebugger
###
###################################################################################################
###################################################################################################
class MyDebugger:

  def open(self):
    import tempfile
    self.s_tmpfile = tempfile.mktemp() + '.log'
    # self.s_value = ''
    if debug: print "[obenDbgFile]: s_tmpfile=%s"%self.s_tmpfile

  def write(self, s):
    if debug: print "[writeDbgFile]: %s"%s
    if hasattr(self,'s_tmpfile'):
      # self.s_value = self.s_value + s + '\n'
      self.s_value = s + '\n'
      fTmp = open(self.s_tmpfile,'a')
      fTmp.write(self.s_value)
      fTmp.close()

  def close(self):
    delattr(self,'s_tmpfile')
    # delattr(self,'s_value')

myDebugger = MyDebugger()


###################################################################################################
###################################################################################################
###
###   class MyClass:
###
###################################################################################################
###################################################################################################
class MyClass:
  pass


###################################################################################################
###################################################################################################
###
###   class MySectionizer
###
###################################################################################################
###################################################################################################
class MySectionizer:

    def __init__(self):
      self.sections = []

    def getLevel(self):
      return len(self.sections)

    def processLevel(self, level):
      # Increase section counter on this level.
      if level == len(self.sections):
        self.sections[level-1] = self.sections[level-1] + 1
      elif level > len(self.sections):
        for i in range(len(self.sections),level):
          self.sections.append(1)
      elif level < len(self.sections):
        for i in range(level,len(self.sections)):
          del self.sections[len(self.sections)-1]
        self.sections[level-1] = self.sections[level-1] + 1

    def __str__(self):
      s = ""
      for i in range(0,len(self.sections)):
        s = s + "%i."%self.sections[i]
      return s

    def clone(self):
      ob = MySectionizer()
      ob.sections = copy.deepcopy(self.sections)
      return ob


###################################################################################################
###################################################################################################
###
###   class MyStack
###
###################################################################################################
###################################################################################################
class MyStack:

    def __init__(self):
        self.clear()
        
    def toString(self):
        s = ''
        for el in self._stack:
            s = s + str(el) + ';'
        return s[:-1]

    def size(self):
        return len(self._stack)

    def clear(self):
        self._stack = []

    def push(self, x):
        self._stack.append(x)

    def pop(self):
        if len(self._stack) > 0:
            return self._stack.pop()
        else:
            return None

    def get(self, i=0):
        if len(self._stack) > 0 and abs(i) < len(self._stack):
            if i < 0:
               return self._stack[len(self._stack)+i-1]
            else:
               return self._stack[i]
        else:
            return None

    def top(self):
        if len(self._stack) > 0:
            return self._stack[len(self._stack)-1]
        else:
            return None

###################################################################################################
