###################################################################################################
# zms.py
#
# Implementation of class ZMS (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTMLFile, ImageFile, MessageDialog, Persistent
from AccessControl.User import UserFolder
from App.Common import package_home
from OFS.Image import Image
from sys import *
import string 
import urllib
import time
# Product imports.
import _sequence
import _mediadb
import _builder
import _cachemanager
import _globals
import _fileutil
import _ftpmanager
import _metadictmanager
import _metacmdmanager
import _metaobjmanager
import _metapatternmanager
import _multilangmanager
import _printablemanager
import _objattrs
import _textformatmanager
import _versionmanager
import _zcatalogmanager
from zmscontainerobject import ZMSContainerObject
from zmscustom import ZMSCustom
from zmsdocument import ZMSDocument
from zmsfile import ZMSFile
from zmsgraphic import ZMSGraphic
from zmslinkcontainer import ZMSLinkContainer
from zmslinkelement import ZMSLinkElement
from zmsnote import ZMSNote
from zmsrubrik import ZMSRubrik
from zmssysfolder import ZMSSysFolder
from zmstable import ZMSTable
from zmsteasercontainer import ZMSTeaserContainer
from zmsteaserelement import ZMSTeaserElement
from zmstextarea import ZMSTextarea


###################################################################################################
###################################################################################################
###   
###   C o m m o n   F u n c t i o n ( s )
###   
###################################################################################################
###################################################################################################

# -------------------------------------------------------------------------------------------------
#  _getNewId
# -------------------------------------------------------------------------------------------------
def _getNewId(_obj, _id):
  for obj in _obj.objectValues(_obj.dGlobalAttrs.keys()):
    try:
      sObjId = str(obj.id)
      iObjId = string.atoi(sObjId[1:])
      if iObjId > _id:
        _id = iObjId
      newid = _getNewId(obj,_id)
      if newid > _id:
        _id = newid
    except:
      pass
  return _id


###################################################################################################
###################################################################################################
###   
###   C o n s t r u c t o r ( s )
###   
###################################################################################################
###################################################################################################

manage_addZMSForm = HTMLFile('manage_addzmsform', globals()) 
def manage_addZMS(self, lang, REQUEST, RESPONSE):
  """ manage_addZMS """
  
  if REQUEST['btn']=='Add':
    
    ##### Add TemplateSet ####
    if len(REQUEST.get('theme','')) > 0:
    
      # Import theme from ZEXP.
      theme = REQUEST['theme']
      theme_zexp = theme + '.zexp'
      _fileutil.importZexp(self,package_home(globals())+'/import/',theme_zexp)
      # Assign folder-id.
      folder_id = REQUEST['folder_id']
      self.manage_renameObject(theme,folder_id)
      homeElmnt = getattr(self,folder_id)
      
    else:
      
      # Add empty folders.
      folder_id = REQUEST['folder_id']
      self.manage_addFolder(folder_id)
      # Common.
      homeElmnt = getattr(self,folder_id)
      homeElmnt.manage_addFolder('common','Global Graphics and Assets')

    ##### Add Zope-Manager ####
    if not homeElmnt.objectValues(['User Folder']):
      user_folder = UserFolder()
      homeElmnt._setObject(user_folder.id, user_folder)
    name = REQUEST.get('manager_name','')
    password = REQUEST.get('manager_password','')
    confirm = REQUEST.get('manager_confirm','')
    roles = ['Manager']
    domains = []
    if len(password) > 0 and len(confirm) > 0 and password == confirm:
        user_folder._addUser(name,password,confirm,roles,domains)
        
    ##### Add ZMS ####
    id = 'content'
    obj = ZMS(id)
    homeElmnt._setObject(obj.id, obj)
    obj = getattr(homeElmnt,obj.id)
        
    ### Init Role-Definitions and Permission Settings.
    obj.initRoleDefs()
    
    ### Init Configuration.
    obj.setConfProperty('ZMSAdministrator.email',REQUEST.get('manager_email',''))
    obj.setConfProperty('ASP.ip_or_domain',REQUEST.get('asp_ip_or_domain',''))
        
    ### Init StyleSheet.
    defaultCSS = ''
    if 'common' in homeElmnt.objectIds(['Folder']):
      if 'defaultCSS' in homeElmnt.common.objectIds(['DTML Method']):
        defaultCSS = homeElmnt.common.defaultCSS.raw
        homeElmnt.common.manage_delObjects(ids=['defaultCSS'])
    obj.getStylesheet(defaultCSS)
        
    ### Init ZCatalog.
    obj.setCatalogPeriod()
    obj.recreateCatalog()
        
    ### Init Languages.
    manage_lang = lang
    obj.autocommit = 1
    obj.setPrimaryLanguage(lang)
    obj.__set_lang__(lang,obj.getLangStr('LANG',lang),'',lang)
        
    ### Init Meta-Dictionaries.
    obj.__init_metadicts__()
    _metadictmanager.initMetaDictAttrs(obj)

    ### Init Object-Attributes.
    obj.synchronizeObjAttrs()
        
    ### Init Version.
    obj.updateVersion(lang,manage_lang,REQUEST)
        
    ##### Object State ####
    obj.setObjStateNew(REQUEST)
    
    ##### Init Properties ####
    obj.setObjProperty('active',1,lang)
    obj.setObjProperty('titleshort','ZMS home',lang)
    obj.setObjProperty('title','ZMS - ZOPE-based contentmanagement system for science, technology and medicine',lang)
    
    ##### VersionManager ####
    obj.onChangeObj(REQUEST,forced=1)
        
    ##### [MODULES] Add Forum / Bulletin Board ####
    if REQUEST.get('forum',0) == 1:
      if _globals.debug: print "[manage_addZMS]: Add Forum / Bulletin Board..."
      # Import special-objects.
      for meta_id in ['discussions_item']:
        xml = open(package_home(globals())+'/import/' + meta_id + '.xml')
        _metaobjmanager.importMetaObj(obj,xml=xml)
        xml.close()
      # Import module.
      id = 'forum'
      obj.initMetapattern(id)
      # Instantiate module.
      _sort_id = 99999
      custom = obj.getMetapattern(id,REQUEST)['title']
      obj.manage_instantiateMetapattern(lang,manage_lang,_sort_id,custom,REQUEST)

    ##### [MODULES] Add Newsletter ####
    if REQUEST.get('newsletter',0) == 1:
      if _globals.debug: print "[manage_addZMS]: Add Newsletter..."
      # Import special-objects.
      for meta_id in ['newsletter_item','newsletter']:
        xml = open(package_home(globals())+'/import/' + meta_id + '.xml')
        _metaobjmanager.importMetaObj(obj,xml=xml)
        xml.close()
      self.setConfProperty('newsletter_item.enabled',0)
      # Import actions.
      for meta_id in ['manage_sendNewsletterItem']:
        xml = open(package_home(globals())+'/import/' + meta_id + '.xml')
        _metacmdmanager.importMetaCmd(obj,xml=xml)
        xml.close()
      # Import module.
      id = 'newsletter'
      obj.initMetapattern(id)
      # Instantiate module.
      _sort_id = 99999
      custom = obj.getMetapattern(id,REQUEST)['title']
      obj.manage_instantiateMetapattern(lang,manage_lang,_sort_id,custom,REQUEST)

    # Return with message.
    message = obj.getLangStr('MSG_INSERTED',manage_lang)%obj.meta_type
    RESPONSE.redirect('%s/%s/manage?manage_tabs_message=%s'%(homeElmnt.absolute_url(),obj.id,urllib.quote(message)))
  
  else:
    RESPONSE.redirect('%s/manage_main'%self.absolute_url())


###################################################################################################
###################################################################################################
###   
###   C l a s s 
###   
###################################################################################################
###################################################################################################
class ZMS(
	ZMSContainerObject,
	_printablemanager.PrintableManager,
	_builder.Builder,
	_cachemanager.CacheManager,
	_ftpmanager.FtpManager,
	_multilangmanager.MultiLanguageManager,
	_metadictmanager.MetadictManager,
	_metacmdmanager.MetacmdManager,
	_metaobjmanager.MetaobjManager,
	_metapatternmanager.MetapatternManager,
	_objattrs.ObjAttrsManager,
	_textformatmanager.TextFormatManager,
	_zcatalogmanager.ZCatalogManager
	):

    # Version-Info.
    # ------------
    zms_version = '1.2'
    zms_build = '102'
    zms_patch = ''

    # Properties.
    # -----------
    meta_type = 'ZMS'
    icon = "misc_/zms/zms.gif"
    icon_disabled = 'misc_/zms/zms_disabled.gif'

    # Management Options.
    # -------------------
    manage_options = (
	{'label': 'TAB_EDIT',         'action': 'manage_main'},
	{'label': 'TAB_PROPERTIES',   'action': 'manage_properties'},
	{'label': 'TAB_ACCESS',       'action': 'manage_users'},
	{'label': 'TAB_IMPORTEXPORT', 'action': 'manage_importexport'},
	{'label': 'TAB_TASKS',        'action': 'manage_tasks'},
	{'label': 'TAB_REFERENCES', 'action': 'manage_RefForm'},
	{'label': 'TAB_HISTORY',      'action': 'manage_UndoForm'},
	{'label': 'TAB_CONFIGURATION','action': 'manage_customizeForm'},
	{'label': 'TAB_PREVIEW',      'action': 'preview_html'}, # empty string defaults to index_html
	)

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = (
		'manage','manage_main',
		'manage_addZMSRubrik','manage_addZMSDocument','manage_addZMSTextarea','manage_addZMSGraphic','manage_addZMSTable','manage_addZMSFile','manage_addZMSTeaserContainer','manage_addZMSNote','manage_addZMSLinkContainer','manage_addZMSLinkElement',
		'manage_deleteObjs','manage_moveObjUp','manage_moveObjDown','manage_cutObjects','manage_copyObjects','manage_pasteObjs',
		'manage_properties','manage_changeProperties',
		'manage_sitemap','manage_search','manage_tasks',
		'manage_rollbackVersionChanges','manage_requestVersionChanges',
		'manage_users',
		)
    __editorPermissions__ = (
		'manage_commitVersionChanges',
		)
    __administratorPermissions__ = (
		'manage_addZMSSysFolder',
		'manage_customizeForm','manage_customizeSystem','manage_updateCatalog',
		'manage_customizeDesignForm','manage_customizeDesign',
		)
    __ac_permissions__=(
		('ZMS Administrator', __administratorPermissions__),
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		('Import/Export objects', ('manage_importexport','manage_import','manage_export',) ),
		# ('Manage users', ('manage_users',) ),
		)

    # Properties.
    # -----------
    __obj_attrs__ = {
        # Changed by
        'change_uid':{'datatype':'LangString','lang_inherit':_globals.LI_FORBIDDEN},
        'change_dt':{'datatype':'LangDateTime','lang_inherit':_globals.LI_FORBIDDEN},
        # Versioned by
        'work_uid':{'datatype':'LangString','lang_inherit':_globals.LI_FORBIDDEN},
        'work_dt':{'datatype':'LangDateTime','lang_inherit':_globals.LI_FORBIDDEN},
        # Properties
        'active':{'datatype':'LangInt','type':'checkbox','lang_inherit':_globals.LI_OBLIGATE},
        'attr_active_start':{'datatype':'LangDateTime','lang_inherit':_globals.LI_OBLIGATE},
        'attr_active_end':{'datatype':'LangDateTime','lang_inherit':_globals.LI_OBLIGATE},
        'title':{'datatype':'LangString','size':40},
        'titleshort':{'datatype':'LangString','size':20},
        'titleimage':{'datatype':'LangImage'},
        'levelnfc':{'datatype':'string'},
        'attr_cacheable':{'datatype':'boolean','default':1},
        # Meta-Data
        'attr_dc_coverage':{'datatype':'string'},
        # Meta-Dictionaries        
        '$metadict':{'datatype':'MetaDict'},
    }
    __extra_obj_attrs__ = {
        # Global-Languages
        'primary_language':{'datatype':'string'},
        'attr_languages':{'datatype':'dictionary'},
        '__attr_dc_metacmds__':{'datatype':'list'},
        '__attr_dc_metaobjs__':{'datatype':'dictionary'},
        '__attr_dc_metadicts__':{'datatype':'list'},
        '__attr_conf_dict__':{'datatype':'dictionary'},
        '__attr_lang_dict__':{'datatype':'dictionary'},
        '__attr_text_formats__':{'datatype':'list'},
        '__attr_text_formats_default__':{'datatype':'string'},
    }

    # Globals.
    # --------
    dGlobalAttrs = {
	'ZMS':{'obj_class':None,'meta_dict':1,'icon':icon,'constructor':None,'obj_attrs':__obj_attrs__},
	'ZMSRubrik':{'obj_class':ZMSRubrik,'meta_dict':1,'icon':ZMSRubrik.icon,'constructor':'manage_addzmsrubrikform','obj_attrs':ZMSRubrik.__obj_attrs__},
	'ZMSDocument':{'obj_class':ZMSDocument,'meta_dict':1,'icon':ZMSDocument.icon,'constructor':'manage_addzmsdocumentform','obj_attrs':ZMSDocument.__obj_attrs__},
	'ZMSTextarea':{'obj_class':ZMSTextarea,'meta_dict':1,'icon':ZMSTextarea.icon,'constructor':'manage_addZMSTextarea','obj_attrs':ZMSTextarea.__obj_attrs__},
	'ZMSGraphic':{'obj_class':ZMSGraphic,'meta_dict':1,'icon':ZMSGraphic.icon,'constructor':'manage_addZMSGraphic','obj_attrs':ZMSGraphic.__obj_attrs__},
	'ZMSTable':{'obj_class':ZMSTable,'meta_dict':0,'icon':ZMSTable.icon,'constructor':'manage_addzmstableform','obj_attrs':ZMSTable.__obj_attrs__},
	'ZMSFile':{'obj_class':ZMSFile,'meta_dict':1,'icon':ZMSFile.icon,'constructor':'manage_addzmsfileform','obj_attrs':ZMSFile.__obj_attrs__},
	'ZMSNote':{'obj_class':ZMSNote,'meta_dict':0,'icon':ZMSNote.icon,'constructor':'manage_addZMSNote','obj_attrs':ZMSNote.__obj_attrs__},
	'ZMSTeaserElement':{'obj_class':ZMSTeaserElement,'meta_dict':0,'icon':ZMSTeaserElement.icon,'constructor':'manage_addZMSTeaserElement','obj_attrs':ZMSTeaserElement.__obj_attrs__},
	'ZMSTeaserContainer':{'obj_class':ZMSTeaserContainer,'meta_dict':0,'icon':ZMSTeaserContainer.icon,'constructor':'manage_addZMSTeaserContainer','obj_attrs':ZMSTeaserContainer.__obj_attrs__},
	'ZMSCustom':{'obj_class':ZMSCustom,'meta_dict':0,'icon':ZMSCustom.icon,'constructor':None,'obj_attrs':ZMSCustom.__obj_attrs__},
	'ZMSSysFolder':{'obj_class':ZMSSysFolder,'meta_dict':1,'icon':ZMSSysFolder.icon,'constructor':'manage_addzmssysfolderform','obj_attrs':ZMSSysFolder.__obj_attrs__},
	'ZMSLinkContainer':{'obj_class':ZMSLinkContainer,'meta_dict':0,'icon':ZMSLinkContainer.icon,'constructor':'manage_addZMSLinkContainer','obj_attrs':ZMSLinkContainer.__obj_attrs__},
	'ZMSLinkElement':{'obj_class':ZMSLinkElement,'meta_dict':1,'icon':ZMSLinkElement.icon,'constructor':'manage_addzmslinkelementform','obj_attrs':ZMSLinkElement.__obj_attrs__},
	}

    # Register DTML-Templates.
    # ------------------------
    index_html = HTMLFile('dtml/zms/index', globals()) # index_html
    f_inactive_html = HTMLFile('dtml/zms/f_inactive', globals()) # inactive_html
    f_headDoctype = HTMLFile('dtml/zms/f_headdoctype', globals()) # Template_L1: DOCTYPE
    f_bodyContent = HTMLFile('dtml/zms/f_bodycontent', globals()) # Template: Body-Content / Element
    f_bodyContent_Float = HTMLFile('dtml/zms/f_bodycontent_float', globals()) # Template: Body-Content / Textkrper
    f_bodyContent_Sitemap = HTMLFile('dtml/zms/f_bodycontent_sitemap', globals()) # Template: Sitemap
    f_bodyContent_Search = HTMLFile('dtml/zms/f_bodycontent_search', globals()) # Template: Search
    f_headTitle = HTMLFile('dtml/zms/f_headtitle', globals()) # Head.Title
    f_headMeta_DC = HTMLFile('dtml/zms/f_headmeta_dc', globals()) # Head.Meta.DC
    headScript = HTMLFile('dtml/zms/headscript', globals()) # Head.Script
    headMeta = HTMLFile('dtml/zms/headmeta', globals()) # Head.Meta
    headCStyleSheet = HTMLFile('dtml/zms/headcstylesheet', globals()) # Template_L1: CSS-Referenz
    f_sitemap = HTMLFile('dtml/zms/f_sitemap', globals()) # f_sitemap
    f_standard_html_request = HTMLFile('dtml/zms/f_standard_html_request', globals()) # f_standard_html_request
    f_standard_html_header = HTMLFile('dtml/zms/f_standard_html_header', globals()) # f_standard_html_header
    f_standard_html_footer = HTMLFile('dtml/zms/f_standard_html_footer', globals()) # f_standard_html_footer
    search_nav_html = HTMLFile('dtml/zms/search_nav', globals()) # search_nav_html


    """
    ###############################################################################################    
    #
    #   CONSTRUCTOR
    #
    ###############################################################################################    
    """
    
    ###############################################################################################
    # ZMS.__init__: 
    #
    # Constructor (initialise a new instance of ZMS).
    ###############################################################################################
    def __init__(self, id): 
      """ ZMS.__init__ """
      self.id = id
      self.logo = ImageFile('Products/zms/images/logo.gif')

    ###############################################################################################
    # ZMS.recurse_updateVersion:
    #
    # Update version.
    ###############################################################################################
    def recurse_updateVersion(self, lang, manage_lang, REQUEST):
      message = ''
      build = getattr(self,'build','000')      

      # Process object.
      # ---------------
      self.getStylesheet()
      self.getSequence()
      self.initRoleDefs()
      self.recurse_updateVersionMetaDicts(lang,manage_lang,REQUEST) # Must be executed first!
      self.recurse_updateVersionMetaObjs(lang,manage_lang,REQUEST)
      self.recurse_updateVersionLanguages(lang,manage_lang,REQUEST)

      # Process super-objects.
      # ----------------------
      message = message + ZMSContainerObject.recurse_updateVersion(self,lang,manage_lang,REQUEST)

      # Return with message.            
      message = message + 'Updated object-attributes from Build #%s to newer Build #%s!'%(build,self.zms_build)
      return message


    # ---------------------------------------------------------------------------------------------
    #  ZMS.getDocumentElement:
    #
    #  The root element of the site.
    # ---------------------------------------------------------------------------------------------
    def getDocumentElement(self):
      return self

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getHome:
    #
    #  Returns the home-folder of the site.
    # ---------------------------------------------------------------------------------------------
    def getHome(self):
      return self.aq_parent
    
    # ---------------------------------------------------------------------------------------------
    #  ZMS.getSequence:
    #
    #  Returns sequence (Sequence).
    # ---------------------------------------------------------------------------------------------
    def getSequence(self):
      id = 'acl_sequence'
      if not id in self.objectIds(['Sequence']):
        _sequence.manage_addSequence(self,startvalue=_getNewId(self,0))
      return getattr(self,id)

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getStylesheet:
    #
    #  Returns stylesheet (DTML Method).
    # ---------------------------------------------------------------------------------------------
    def getStylesheet(self, defaultCSS=''):
      id = 'stylesheet.css'
      if not id in self.objectIds(['DTML Method']):
        title = '*** DO NOT DELETE OR MODIFY ***'
        if len(defaultCSS) == 0:
          defaultCSS = HTMLFile('dtml/zms/__init__stylesheet', globals())(self)
        raw = getattr(self,'stylesheet',defaultCSS)
        self.manage_addDTMLMethod(id,title,raw)
        try: delattr(self,'stylesheet')
        except: pass
      return getattr(self,id)

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getNewId:
    #
    #  Returns new (unique) Object-ID.
    # ---------------------------------------------------------------------------------------------
    def getNewId(self, id_prefix='e'): 
      return '%s%i'%(id_prefix,self.getSequence().nextVal())

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getMetaTypes:
    #
    #  Returns list of meta-types.
    # ---------------------------------------------------------------------------------------------
    def getMetaTypes(self, REQUEST): 
      meta_types = []
      for meta_type in self.dGlobalAttrs.keys():
        if meta_type not in ['ZMS','ZMSCustom']:
          meta_types.append((self.display_type(REQUEST,meta_type),meta_type))
      for meta_id in self.getMetaobjIds():
        meta_types.append((self.display_type(REQUEST,meta_id),meta_id))
      meta_types.sort()
      return meta_types

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getDCCoverage
    # ---------------------------------------------------------------------------------------------
    def getDCCoverage(self, REQUEST): 
      return 'obligation'

    # ---------------------------------------------------------------------------------------------
    #  ZMS.sendMail:
    #
    #  Sends Mail via MailHost.
    # ---------------------------------------------------------------------------------------------
    def sendMail(self, mto, msubject, mbody, REQUEST):
    
      ##### Get Sender ####
      auth_user = REQUEST['AUTHENTICATED_USER']
      mfrom = str(getattr(auth_user,'email',self.getConfProperty('ZMSAdministrator.email','')))
    
      ##### Get MailHost ####
      mailhost = None
      homeElmnt = self.getHome()
      if len(homeElmnt.objectValues(['Mail Host'])) == 1:
        mailhost = homeElmnt.objectValues(['Mail Host'])[0]
      elif getattr(homeElmnt,'MailHost',None) is not None:
        mailhost = getattr(homeElmnt,'MailHost',None)

      ##### Get MessageText ####
      messageText = ''
      messageText = messageText + 'From: %s\n'%mfrom
      messageText = messageText + 'To: %s\n'%mto
      messageText = messageText + 'Subject: %s\n'%msubject
      messageText = messageText + mbody
      if _globals.debug: print "[sendMail]: %s"%messageText

      ##### Send mail ####
      if mailhost is not None and len(mto) > 0 and len(mfrom) > 0:
        try:
          mailhost.send(messageText)
        except:
          pass



    """
    ###############################################################################################    
    #
    #   ZMS - System Configuration
    #
    ###############################################################################################    
    """

    # ---------------------------------------------------------------------------------------------
    #  ZMS.__get_attr_conf_dict__:
    #
    #  Returns property from configuration.
    # ---------------------------------------------------------------------------------------------
    def __get_attr_conf_dict__(self):
      return getattr(self,'__attr_conf_dict__',{})

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getConfProperty:
    #
    #  Returns property from configuration.
    # ---------------------------------------------------------------------------------------------
    def getConfProperty(self, key, default=None):
      value = default
      attr_conf_dict = self.__get_attr_conf_dict__()
      if attr_conf_dict.has_key(key):
        value = attr_conf_dict[key]
      #++ print '[%s.getConfProperty]: %s=%s'%(self.meta_type,key,str(value))
      return value

    # ---------------------------------------------------------------------------------------------
    #  ZMS.setConfProperty:
    #
    #  Returns property from configuration.
    # ---------------------------------------------------------------------------------------------
    def setConfProperty(self, key, value):
      attr_conf_dict = self.__get_attr_conf_dict__()
      attr_conf_dict[key] = value
      #++ print '[%s.setConfProperty]: %s=%s'%(self.meta_type,key,str(value))
      self.__attr_conf_dict__ = attr_conf_dict
      self.__attr_conf_dict__ = self.__attr_conf_dict__.copy()

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getMediaDb:
    #
    #  Returns mediadb (MediaDb).
    # ---------------------------------------------------------------------------------------------
    def getMediaDb(self):
      id = 'acl_mediadb'
      return getattr(self,id,None)

    # ---------------------------------------------------------------------------------------------
    #	ZMS.getAutocommit
    # ---------------------------------------------------------------------------------------------
    def getAutocommit(self): 
      return getattr(self,'autocommit',0)

    # ---------------------------------------------------------------------------------------------
    #  ZMS.doAutocommit
    #
    #  Auto-Commit changes.
    # ---------------------------------------------------------------------------------------------
    def doAutocommit(self): 
      _versionmanager.doAutocommit(self)

    # ---------------------------------------------------------------------------------------------
    #	ZMS.doAutopack:
    #
    #	Auto-Pack ZODB.
    # ---------------------------------------------------------------------------------------------
    def doAutopack(self, lang, manage_lang, REQUEST):
      message = ''
      threshold = getattr(self,'property_threshold',None)
      days = getattr(self,'property_days',None)
      if threshold is not None and days is not None:
        undoable_transactions = self.aq_parent.undoable_transactions(first_transaction=None,last_transaction=None,PrincipiaUndoBatchSize=999)
        transactions = len(undoable_transactions)
        #++ print "[%s.doAutopack]: Transactions: %i (>=Threshold: %i?)"%(self.meta_type,transactions,threshold)
        if transactions >= threshold:
	  t0 = undoable_transactions[-1]['time'].timeTime()+days*86400.0
	  t1 = time.time()
	  #++ print "[%s.doAutopack]: Next Scheduled Time: %s"%(self.meta_type,self.getLangFmtDate(t0,manage_lang))
	  #++ print "[%s.doAutopack]: Current Time: %s"%(self.meta_type,self.getLangFmtDate(t1,manage_lang))
          if t1 >= t0:
            #++ print "[%s.doAutopack]: Remove previous revisions of objects that are older than %i days."%(self.meta_type,days)
            try:
              self.Control_Panel.Database.manage_pack(days)
            except:
              pass
      return message


    ###############################################################################################
    # ZMS.manage_customizeSystem: 
    #
    # Change system properties.
    ###############################################################################################
    def manage_customizeSystem(self, btn, key, lang, manage_lang, REQUEST, RESPONSE): 
      """ ZMS.manage_customizeSystem """

      message = ''
      
      ##### ASP ####
      if key == 'ASP':
        self.setConfProperty('ZMSAdministrator.email',REQUEST.get('zmsadministrator_email',''))
        self.setConfProperty('ASP.ip_or_domain',REQUEST.get('asp_ip_or_domain',''))
        self.setConfProperty('ASP.login',REQUEST.get('asp_login',''))
        if len(REQUEST.get('asp_password','')) > 0 and REQUEST.get('asp_password','') == REQUEST.get('asp__confirm',''):
          self.setConfProperty('ASP.password',REQUEST.get('asp_password',''))
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      ##### AutoCommit ####
      elif key == 'AutoCommit':
        old_autocommit = self.getAutocommit()
        new_autocommit = REQUEST.get('autocommit',0)
        self.autocommit = new_autocommit
        if old_autocommit == 0 and new_autocommit == 1:
          self.doAutocommit()
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      ##### Version ####
      elif key == 'Version':
        if btn == 'Regenerate':
          message = self.recurse_updateVersion(lang,manage_lang,REQUEST)
      
      ##### Clients ####
      elif key == 'Clients':
        if btn == 'Change':
          self.setConfProperty('Portal.Master',REQUEST.get('portal_master',''))
          l = []
          for s in string.split(REQUEST.get('portal_clients',''),'\n'):
            s = string.strip(s)
            if len(s) > 0:
              l.append(s)
          self.setConfProperty('Portal.Clients',l)
          message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      ##### AutoPack ####
      elif key == 'AutoPack':
        if btn == 'Change':
          try:      
            self.property_threshold = int(REQUEST['property_threshold'])
            self.property_days = int(REQUEST['property_days'])
          except:
            self.property_threshold = None
            self.property_days = None
        elif btn == 'Clear':
          self.property_threshold = None
          self.property_days = None
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      ##### MediaDb ####
      elif key == 'MediaDb':
        if btn == 'Create':
          location = string.strip(REQUEST['mediadb_location'])
          _mediadb.manage_addMediaDb(self,location)
        elif btn == 'Remove':
          self.manage_delObjects(['acl_mediadb'])
        message = self.getLangStr('MSG_CHANGED',manage_lang)
          
      ##### InstalledProducts ####
      elif key == 'InstalledProducts':
        if btn == 'Change':
          self.setConfProperty('Demo.Active',REQUEST.get('demo_active',0))
          for product in ['jdk12','pil','htmldoc']:
            self.setConfProperty('%s.%s'%(key,product),REQUEST.get(product,None))
          self.setConfProperty('InstalledProducts.pil.scale',REQUEST.get('pil_scale',0))
          self.setConfProperty('InstalledProducts.pil.thumbnail',REQUEST.get('pil_thumbnail',0))
          self.setConfProperty('InstalledProducts.pil.thumbnail.max',REQUEST.get('pil_thumbnail_max',100))
          message = self.getLangStr('MSG_CHANGED',manage_lang)
          
      ##### StandardObjects ####
      elif key == 'StandardObjects':
        meta_types = []
        meta_types.extend(self.dGlobalAttrs.keys())
        meta_types.extend(self.getMetaobjIds())
        for meta_type in meta_types:
          self.setConfProperty('%s.enabled'%meta_type,meta_type in REQUEST.get('meta_types',[]))
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      # Return with message.
      message = urllib.quote(message)
      return RESPONSE.redirect('manage_customizeForm?lang=%s&manage_lang=%s&manage_tabs_message=%s#_%s'%(lang,manage_lang,message,key))


    """
    ###############################################################################################    
    #
    #   ZMS - Versions
    #
    ###############################################################################################    
    """

    # ---------------------------------------------------------------------------------------------
    #	ZMS.updateVersion:
    #
    #	Update version.
    # ---------------------------------------------------------------------------------------------
    def updateVersion(self, lang, manage_lang, REQUEST):
      #++ if _globals.debug: print "[%s.updateVersion]"%(self.meta_type)
      message = ''
      build = getattr(self,'build','000')
      if build < self.zms_build:
        message = self.recurse_updateVersion(lang,manage_lang,REQUEST)
        setattr(self,'build',self.zms_build)
      message = message + self.doAutopack(lang,manage_lang,REQUEST)
      return message


    """
    ###############################################################################################    
    #
    #   Customize
    #
    ###############################################################################################    
    """

    # Customize Interfaces.
    manage_customizeForm = HTMLFile('dtml/zms/manage_customizeform', globals()) 

    ###############################################################################################
    # ZMS.manage_customizeDesign: 
    #
    # Change layout (CSS Stylesheet and Logo).
    ###############################################################################################
    manage_customizeDesignForm = HTMLFile('dtml/zms/manage_customizedesignform', globals()) 
    def manage_customizeDesign(self, lang, manage_lang, REQUEST, RESPONSE):
      """ ZMS.manage_customizeDesign """
      
      if REQUEST['btn'] == self.getLangStr('BTN_CHANGE',manage_lang):
      
        # Change Stylesheet.
        # ------------------
        if REQUEST.has_key('stylesheet'):
          method = self.getStylesheet()
          data = REQUEST['stylesheet']
          title = method.title
          self.getStylesheet().manage_edit(data,title)
          
        # Change Layout of Sitemap.
        # -------------------------
        if REQUEST.has_key('attr_layoutsitemap'):
          if len(REQUEST['attr_layoutsitemap'])>0:
            self.attr_layoutsitemap = int(REQUEST['attr_layoutsitemap'])
          elif hasattr(self,'attr_layoutsitemap'):
            delattr(self,'attr_layoutsitemap')
      
      # Upload Logo.
      # ------------      
      elif REQUEST['btn'] == self.getLangStr('BTN_UPLOAD',manage_lang):
        self.logo = Image(id='logo',title='',file='')
        self.logo.manage_upload(REQUEST['file'],REQUEST)

      # Reset Logo.
      # -----------      
      elif REQUEST['btn'] == self.getLangStr('BTN_CLEAR',manage_lang):
        self.logo = ImageFile('Products/zms/images/logo.gif')
      
      # Return with message.
      message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))
      return RESPONSE.redirect('manage_customizeDesignForm?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))


    """
    ###############################################################################################
    ###  
    ###  DOM-Methoden 
    ### 
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #	ZMS.getParentNode:
    #
    #	The parent of this node. 
    #	All nodes except root may have a parent.
    # ---------------------------------------------------------------------------------------------
    def getParentNode(self): 
      return None


    """
    ###############################################################################################    
    #
    #   BUILDER
    #
    ###############################################################################################    
    """

    ###############################################################################################
    # ZMS.xmlOnStartElement(self, sTagName, dAttrs, oParentNode, oRoot):
    #
    # handler for XML-Builder (_builder.py)
    ###############################################################################################
    def xmlOnStartElement(self, sTagName, dTagAttrs, oParentNode, oRoot):
      """ZMS.xmlOnStartElement"""
      _globals.myDebugger.write("[%s.xmlOnStartElement]: sTagName=%s"%(self.meta_type,sTagName))

      # remove all ZMS-objects.
      self.manage_delObjects(self.objectIds(self.dGlobalAttrs.keys()))
      # remove all languages.
      for s_lang in self.getLanguages():
        self.__del_lang__(s_lang)
      
      self.dTagStack = _globals.MyStack()
      self.dValueStack  = _globals.MyStack()
      
      # WORKAROUND! The member variable "aq_parent" does not contain the right parent object at 
      # this stage of the creation process (it will later on!). Therefore, we introduce a special
      # attribute containing the parent object, which will be used by xmlGetParent() (see below).
      self.oParent = None

###################################################################################################
